# Generated by roxygen2: do not edit by hand

S3method(chart_ax_x,default)
S3method(chart_ax_x,ms_chart_ex)
S3method(chart_ax_y,default)
S3method(chart_ax_y,ms_chart_ex)
S3method(chart_data_fill,default)
S3method(chart_data_fill,ms_chart_ex)
S3method(chart_data_labels,default)
S3method(chart_data_labels,ms_chart_ex)
S3method(chart_data_stroke,default)
S3method(chart_data_stroke,ms_chart_ex)
S3method(chart_extra_parts,ms_chart)
S3method(chart_extra_parts,ms_chart_ex)
S3method(chart_graphicdata_uri,ms_chart)
S3method(chart_graphicdata_uri,ms_chart_ex)
S3method(chart_labels_text,default)
S3method(chart_labels_text,ms_chart_ex)
S3method(chart_part_content_type,ms_chart)
S3method(chart_part_content_type,ms_chart_ex)
S3method(chart_part_rel_type,ms_chart)
S3method(chart_part_rel_type,ms_chart_ex)
S3method(chart_reference_xml,ms_chart)
S3method(chart_reference_xml,ms_chart_ex)
S3method(chart_settings,ms_areachart)
S3method(chart_settings,ms_barchart)
S3method(chart_settings,ms_boxplotchart)
S3method(chart_settings,ms_bubblechart)
S3method(chart_settings,ms_chart_ex)
S3method(chart_settings,ms_linechart)
S3method(chart_settings,ms_paretochart)
S3method(chart_settings,ms_piechart)
S3method(chart_settings,ms_radarchart)
S3method(chart_settings,ms_scatterchart)
S3method(chart_settings,ms_stockchart)
S3method(cx_fill_cat_column,default)
S3method(cx_fill_cat_column,ms_histogramchart)
S3method(cx_fill_cat_column,ms_paretochart)
S3method(cx_fill_cat_column,ms_sunburstchart)
S3method(cx_fill_cat_column,ms_treemapchart)
S3method(cx_uses_connectors,default)
S3method(cx_uses_connectors,ms_waterfallchart)
S3method(format,ms_boxplotchart)
S3method(format,ms_chart)
S3method(format,ms_funnelchart)
S3method(format,ms_histogramchart)
S3method(format,ms_paretochart)
S3method(format,ms_sunburstchart)
S3method(format,ms_treemapchart)
S3method(format,ms_waterfallchart)
S3method(ph_with,ms_chart)
S3method(print,ms_boxplotchart)
S3method(print,ms_chart)
S3method(print,ms_funnelchart)
S3method(print,ms_histogramchart)
S3method(print,ms_paretochart)
S3method(print,ms_sunburstchart)
S3method(print,ms_treemapchart)
S3method(print,ms_waterfallchart)
S3method(sheet_add_drawing,ms_chart)
S3method(to_pml,date_ref)
S3method(to_pml,label_ref)
S3method(to_pml,labels_options)
S3method(to_pml,ms_areachart)
S3method(to_pml,ms_barchart)
S3method(to_pml,ms_bubblechart)
S3method(to_pml,ms_linechart)
S3method(to_pml,ms_piechart)
S3method(to_pml,ms_radarchart)
S3method(to_pml,ms_scatterchart)
S3method(to_pml,ms_stockchart)
S3method(to_pml,num_ref)
S3method(to_pml,str_ref)
S3method(update,col_ref)
export(as_bar_stack)
export(body_add_chart)
export(chart_ax_x)
export(chart_ax_y)
export(chart_data_fill)
export(chart_data_labels)
export(chart_data_line_style)
export(chart_data_line_width)
export(chart_data_size)
export(chart_data_smooth)
export(chart_data_stroke)
export(chart_data_symbol)
export(chart_fill_ggplot2)
export(chart_labels)
export(chart_labels_text)
export(chart_settings)
export(chart_table)
export(chart_theme)
export(ms_areachart)
export(ms_barchart)
export(ms_boxplotchart)
export(ms_bubblechart)
export(ms_chart_combine)
export(ms_funnelchart)
export(ms_histogramchart)
export(ms_linechart)
export(ms_paretochart)
export(ms_piechart)
export(ms_radarchart)
export(ms_scatterchart)
export(ms_stockchart)
export(ms_sunburstchart)
export(ms_treemapchart)
export(ms_waterfallchart)
export(mschart_theme)
export(set_theme)
export(theme_ggplot2)
importFrom(cellranger,as.cell_limits)
importFrom(cellranger,as.range)
importFrom(cellranger,cell_limits)
importFrom(cellranger,ra_ref)
importFrom(cellranger,to_string)
importFrom(data.table,":=")
importFrom(data.table,.N)
importFrom(data.table,as.data.table)
importFrom(data.table,dcast.data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,setDF)
importFrom(data.table,setnames)
importFrom(data.table,setorderv)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colors)
importFrom(grDevices,rgb)
importFrom(htmltools,htmlEscape)
importFrom(officer,add_slide)
importFrom(officer,body_add_xml)
importFrom(officer,fp_border)
importFrom(officer,fp_text)
importFrom(officer,ph_location_fullsize)
importFrom(officer,ph_with)
importFrom(officer,read_pptx)
importFrom(officer,sheet_add_drawing)
importFrom(officer,sheet_write_data)
importFrom(officer,styles_info)
importFrom(officer,to_pml)
importFrom(officer,uuid_generate)
importFrom(officer,xlsx_drawing)
importFrom(scales,hue_pal)
importFrom(stats,as.formula)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(utils,browseURL)
importFrom(writexl,write_xlsx)
importFrom(xml2,"xml_attr<-")
importFrom(xml2,as_xml_document)
importFrom(xml2,read_xml)
importFrom(xml2,write_xml)
importFrom(xml2,xml_add_child)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_find_first)
importFrom(xml2,xml_remove)
importFrom(xml2,xml_replace)
