#' Index of US Industrial Production
#'
#' Index of US industrial production (1985 = 100).
#'
#' This is a transformation of the `USProdIndex` data
#' from package 'AER' into a simple data.frame.
#'
#' @name us_indus_prod
#' @format A data frame with 256 rows and 3 variables
NULL

#' Dummy dataset for barchart
#'
#' A dataset containing two categorical variables and one integer variable:
#'
#' * browser web browser
#' * serie id of series
#' * value integer values
#'
#' @name browser_data
#' @format A data frame with 18 rows and 3 variables
NULL

#' Dummy dataset for time series charts
#'
#' A dataset containing a date, a categorical, and an integer variable:
#'
#' * date date values
#' * browser web browser
#' * freq values in percent
#'
#' @name browser_ts
#' @format A data frame with 36 rows and 3 variables
NULL
