st_layouttarget <- c('inner', 'outer')

st_layoutmode <- c('edge', 'factor')

st_sizerepresents <- c('area', 'w')

st_splittype <- c('auto', 'cust', 'percent', 'pos', 'val')

st_lblalgn <- c('ctr', 'l', 'r')

st_dlblpos <- c(
  'bestFit',
  'b',
  'ctr',
  'inBase',
  'inEnd',
  'l',
  'outEnd',
  'r',
  't'
)

st_markerstyle <- c(
  'circle',
  'dash',
  'diamond',
  'dot',
  'none',
  'plus',
  'square',
  'star',
  'triangle',
  'x',
  'auto'
)

st_linestyle <- c('none', 'solid', 'dotted', 'dashed')

st_trendlinetype <- c('exp', 'linear', 'log', 'movingAvg', 'poly', 'power')

st_errdir <- c('x', 'y')

st_errbartype <- c('both', 'minus', 'plus')

st_errvaltype <- c('cust', 'fixedVal', 'percentage', 'stdDev', 'stdErr')

st_grouping <- c('percentStacked', 'standard', 'stacked')

st_scatterstyle <- c(
  'none',
  'line',
  'lineMarker',
  'marker',
  'smooth',
  'smoothMarker'
)

st_radarstyle <- c('standard', 'marker', 'filled')

st_bargrouping <- c('percentStacked', 'clustered', 'standard', 'stacked')

st_bardir <- c('bar', 'col')

st_shape <- c('cone', 'coneToMax', 'box', 'cylinder', 'pyramid', 'pyramidToMax')

st_ofpietype <- c('pie', 'bar')

st_axpos <- c('b', 'l', 'r', 't')

st_crosses <- c('autoZero', 'max', 'min')

st_crossbetween <- c('between', 'midCat')

st_tickmark <- c('cross', 'in', 'none', 'out')

st_ticklblpos <- c('high', 'low', 'nextTo', 'none')

st_timeunit <- c('days', 'months', 'years')

st_builtinunit <- c(
  'hundreds',
  'thousands',
  'tenThousands',
  'hundredThousands',
  'millions',
  'tenMillions',
  'hundredMillions',
  'billions',
  'trillions'
)

st_pictureformat <- c('stretch', 'stack', 'stackScale')

st_orientation <- c('maxMin', 'minMax')

st_legendpos <- c('b', 'tr', 'l', 'r', 't', 'n')

st_dispblanksas <- c('span', 'gap', 'zero')

st_pagesetuporientation <- c('default', 'portrait', 'landscape')
