% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes.R
\name{chart_fill_ggplot2}
\alias{chart_fill_ggplot2}
\title{Apply ggplot2 color scale}
\usage{
chart_fill_ggplot2(x, stroke = TRUE)
}
\arguments{
\item{x}{a mschart object}

\item{stroke}{a boolean. Apply the color scale to stroke? Defaults to \code{TRUE}.}
}
\value{
a mschart object
}
\description{
The default hue color scale from ggplot2.
}
\section{chart_fill_ggplot2()}{


\if{html}{\figure{fig_theme_ggplot2.png}{options: width="500"}}
}

\examples{
p <- ms_scatterchart(
  data = iris, x = "Sepal.Length",
  y = "Sepal.Width", group = "Species"
)

p <- theme_ggplot2(p)
p <- chart_fill_ggplot2(p)
}
